//ATtiny25 code
//Reads in voltage values on PB2 and converts A to D
//Outputs thresholded signal on PB1 and Manchester encoded signal on PB3
//Watchdog timer for power down mode with power down control signal to other ICs on PB4
//106 kHz CLK on PB0 for testing purposes
//Zi Ling Kang (zk29)
//Hemanshu Chawda (hkc2)

//----------------INCLUDEs------------------------
#include <tiny25.h>
#include <stdlib.h>
#include <delay.h>
#include <sleep.h>

//----------------GLOBAL-VARIABLES------------------
int khz_count = 0;            // Timer counter
unsigned char count = 0x00;   // Timer counter
unsigned char sec_counter = 0x00;
// LED 3 and 4 are switched
// need to generate a sine wave at 4.068 MHz
// http://instruct1.cit.cornell.edu/courses/ee476/labs/s2007/lab2.html
unsigned char adcval = 0x00;
unsigned char adcthres = 0x00;
unsigned char adcmax = 0x00;
unsigned char adcmin = 0xff;
unsigned char clk106k = 0x0;

// -----------------ISRs------------------------------------------
interrupt [TIM0_COMPA] void timer0_compareA()
{
    //find min and set threshold
    if (adcval < adcmin){
      adcmin = adcval;
      adcthres = (adcmax-adcmin)>>1;
    }
    //find max and set threshold
    if (adcval > adcmax){
      adcmax = adcval;
      adcthres = (adcmax-adcmin)>>1;
    }
    //toggle PB3 based on ADC value and threshold
    if (adcval <= adcthres){
      PORTB.1 = 0x0;
      PORTB.3 = clk106k;
    } else{
      PORTB.1 = 0x1;
      PORTB.3 = ~clk106k;
    }
   PORTB.0 = ~PORTB.0;
   clk106k = ~clk106k;
   // This interrupt service routine is called when timer 0 overflows.
   // Since prescalar is chosen to be 1, timer compares at frequency of 16MHz/(1 prescalar * 77 compare value) = 212.779kHz
   if(khz_count == 1023){   // if khz_count has counted another khz, check to see if count has reached 212
      khz_count = 0;
      if(count == 212){     // if count has counted to 212, increment the second counter
        count = 0;
        sec_counter++;
        // toggle the LEDs hopefully every second
        }
        else count++;       // if khz_count has counted another khz but count isnt 212, increment count
      }
      else khz_count++;     // if khz_count hasn't reached 1023, just increment khz_count
}

interrupt [WDT] void watchdog_timeout()
{
   // This interrupt service routine is called when the Watchdog Timer Timeouts
}

// -----------MAIN--------------------------------------------------
int main(){
   // set port B: Output - PB0, PB1; Input - PB2-PB5
   DDRB = 0b111011;

   TCNT0 = 0; // clear timer counter
   TCCR0A = 0b00000010;
   TCCR0B = 0b00000001; // prescalar = 1 (full clock rate)
   OCR0A = 76;  // compare value of 76 for timer 0 compare -> generate 106kHz square wave
                        // since 0 indexed, divide 16MHz/77 -> 212khz switching -> 106kHz square wave
   TIMSK = 0b00010000;  // enable timer0a compare match interrupt

   // ADC
   ADMUX = 0b00100001; // Vcc used as Vref-, right adjusted, MUX = 0001 to select PB2
   ADCSRA = 0b11000101; // Enable ADC, start first conversion, and set prescalar to 16

   // Wait for conversion to complete
   while( ADCSRA.6 == 0b1)
   { }

   // Power Reduction Register
   PRR = 0b00001010; // Turn off the timer1 and USI (universal serial interface) to save power

   // MCU Control Register
   MCUCR = 0b10110100; // diable BOD during sleep and enable sleep to go into power-down mode

   // Watchdog Timer Control Register
   WDTCR = 0b01111001; // Watchdog Timeout Interrupt Enable and Change Enable and Watchdog Enable
   // set to timeout every 1024k cycles at 128kHz = 8 seconds  but for some reason goes 16 seconds...
   PORTB.4 = 0x0;

   // enable interrupts
   #asm("sei");

   sleep_enable();
   while (1){

  	PORTB.4 = 0x1;        //power down signal high when not in power down mode
    // Start the conversion
    ADCSRA.6 = 0b1;

    // Wait for conversion to complete
    while( ADCSRA.6 == 0b1)
    { }

    adcval = ADCH;  //get the 8 MSBs of the ADC

/*  //find min and set threshold
    if (adcval < adcmin){
      adcmin = adcval;
      adcthres = (adcmax-adcmin)>>1;
    }
    //find max and set threshold
    if (adcval > adcmax){
      adcmax = adcval;
      adcthres = (adcmax-adcmin)>>1;
    }
    //toggle PB1 based on ADC value and threshold, Manchester encoding on PB3
    if (adcval <= adcthres){
      PORTB.1 = 0x0;
      PORTB.3 = clk106k;  
    } else{
      PORTB.1 = 0x1;
      PORTB.3 = ~clk106k;
    }
*/
    //reset values and power down
    if (sec_counter == 8){
      adcmax = 0x00;
      adcmin = 0xff;
      PORTB.1 = 0x0;
      PORTB.3 = 0x0;
      PORTB.4 = 0x0;         //power down signal
      powerdown();           //power down ATtiny25 to save power
      }
    }
 return 0;  // never happens but gets rid of warning!
}
